/////////////////////////////////////////////////////////////////////////////
//
//  WinAPI_24.cpp
//
//  (C) Radek Chalupa, 2002
//
/////////////////////////////////////////////////////////////////////////////

#ifndef WINVER
#define WINVER 0x0400
#endif

#ifndef _WIN32_WINNT
#define _WIN32_WINNT 0x0400
#endif						

#ifndef _WIN32_WINDOWS
#define _WIN32_WINDOWS 0x0410
#endif

#ifndef _WIN32_IE
#define _WIN32_IE 0x0500
#endif

#include <windows.h>
#include <tchar.h>
#include "resource.h"

HINSTANCE g_hInst;
HWND g_hWnd;
const TCHAR chClassName[] = TEXT("NotifyIconWindowsClass");
const TCHAR chAppName[] = TEXT("NotifyIcon demo");

#define WM_NOTIFY_ICON  ( WM_APP + 1 )

BOOL DeleteNotifyIcon();

/////////////////////////////////////////////////////////////////////////////
// Procedura okna 

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch ( uMsg ) 
	{
		case WM_CLOSE:
			DeleteNotifyIcon();
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_NOTIFY_ICON:
			switch (lParam)
			{
				case WM_LBUTTONDBLCLK:
					ShowWindow(hWnd, SW_RESTORE);
					SetForegroundWindow(hWnd);
					break;
			}
			break;
		case WM_SYSCOMMAND:
			switch ( wParam )
			{
				case SC_MINIMIZE:
					ShowWindow(hWnd, SW_HIDE);
					return 0;
			}
			break;
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

/////////////////////////////////////////////////////////////////////////////
// Pidn ikony do oznamovac oblasti

BOOL AddNotifyIcon()
{
	NOTIFYICONDATA nid;
	ZeroMemory(&nid, sizeof(nid));
	nid.cbSize = sizeof(NOTIFYICONDATA);
	nid.hWnd = g_hWnd;
	nid.hIcon = (HICON)GetClassLongPtr(g_hWnd, GCLP_HICONSM);
	if ( !nid.hIcon )
		return FALSE;
	lstrcpyn(nid.szTip, chAppName, sizeof(nid.szTip));
	nid.uID = 1;
	nid.uCallbackMessage = WM_NOTIFY_ICON;
	
#if _WIN32_IE >= 0x0500 
	nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP | NIF_INFO;
	nid.dwInfoFlags = NIIF_INFO;
	nid.dwState = 0;
	nid.uVersion = NOTIFYICON_VERSION; 
	nid.uTimeout = 10000;
	lstrcpy(nid.szInfo, TEXT("Tady me bt\nnjak text do 255 znak\n:-))"));
	lstrcpy(nid.szInfoTitle, chAppName);
#else 
	nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
#endif 
	if ( !Shell_NotifyIcon(NIM_ADD, &nid) )
	{
		MessageBox(g_hWnd, TEXT("Nelze pidat ikonu"), NULL, MB_ICONERROR | MB_TASKMODAL);
		return FALSE;
	}
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// Odebrn ikony z oznamovac oblasti

BOOL DeleteNotifyIcon()
{
	NOTIFYICONDATA nid;
	ZeroMemory(&nid, sizeof(nid));
	nid.cbSize = sizeof(NOTIFYICONDATA);
	nid.hWnd = g_hWnd;
	nid.uID = 1;
	if ( !Shell_NotifyIcon(NIM_DELETE, &nid) )
	{
		MessageBox(g_hWnd, TEXT("Nelze odebrat ikonu"), NULL, MB_ICONERROR | MB_TASKMODAL);
		return FALSE;
	}
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// Registrace tdy okna a vytvoen okna

BOOL InitApp()
{
	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX); 
	wcex.style = CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc = (WNDPROC)WndProc;
	wcex.cbClsExtra = 0;
	wcex.cbWndExtra = 0;
	wcex.hInstance = g_hInst;
	wcex.hIcon = LoadIcon(g_hInst, MAKEINTRESOURCE(IDR_MAIN));
	wcex.hIconSm = LoadIcon(g_hInst, MAKEINTRESOURCE(IDR_MAIN));
	wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground = GetSysColorBrush(COLOR_WINDOW);
	wcex.lpszMenuName = NULL;
	wcex.lpszClassName = chClassName;
	if ( !RegisterClassEx(&wcex) )
		return FALSE;
	g_hWnd = CreateWindowEx(0, chClassName, chAppName,
		WS_OVERLAPPEDWINDOW,
		100, 50, 400, 300,
		NULL, NULL, g_hInst, NULL);
	if ( !g_hWnd )
		return FALSE;
	if ( !AddNotifyIcon() )
		return FALSE;
	ShowWindow(g_hWnd, SW_SHOW);
	UpdateWindow(g_hWnd);
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// Hlavn funkce programu pro Windows

int WINAPI _tWinMain(HINSTANCE hInstance,	HINSTANCE hPrevInstance,
	LPTSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
	g_hInst = hInstance;
	if ( !InitApp() ) 
		return -1;
	// Hlavn smyka zprv:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return msg.wParam;
}
